@@ -62,6 +62,7 @@ class DisclosureSystem:
         self.validateEntryText = False
         self.allowedExternalHrefPattern = None
         self.allowedImageTypes = None
+        self.arcroleCyclesAllowed = {}
         self.schemaValidateSchema = None
         self.blockDisallowedReferences = False
         self.maxSubmissionSubdirectoryEntryNesting = 0
@@ -182,6 +183,10 @@ class DisclosureSystem:
                                     self.allowedExternalHrefPattern = re.compile(dsElt.get("allowedExternalHrefPattern"))
                                 if dsElt.get("allowedImageTypes"):
                                     self.allowedImageTypes = json.loads(dsElt.get("allowedImageTypes"))
+                                if dsElt.get("arcroleCyclesAllowed"):
+                                    self.arcroleCyclesAllowed = json.loads(dsElt.get("arcroleCyclesAllowed"))
+                                    for arcrole, specSet in self.arcroleCyclesAllowed.items():
+                                        self.arcroleCyclesAllowed[arcrole] = tuple(specSet)
                                 self.blockDisallowedReferences = dsElt.get("blockDisallowedReferences") == "true"
                                 try:
                                     self.maxSubmissionSubdirectoryEntryNesting = int(dsElt.get("maxSubmissionSubdirectoryEntryNesting"))
@@ -396,7 +401,7 @@ class DisclosureSystem:
                                              level=logging.ERROR)
             etree.clear_error_log()
 
-    def mappedUrl(self, url):
+    def mappedUrl(self, url: str) -> str:
         if url in self.mappedFiles:
             mappedUrl = self.mappedFiles[url]
         else:  # handle mapped paths
@@ -407,6 +412,15 @@ class DisclosureSystem:
                     break
         return mappedUrl
 
+    def isMappedUrl(self, url: str) -> bool:
+        if url in self.mappedFiles:
+            return True
+        else:  # handle mapped paths
+            for mapFrom, mapTo in self.mappedPaths:
+                if url.startswith(mapFrom):
+                    return True
+        return False
+
     def uriAuthorityValid(self, uri):
         if self.standardTaxonomiesUrl:
             return UrlUtil.authority(uri) in self.standardAuthorities
@@ -429,7 +443,7 @@ class DisclosureSystem:
             return href in self.standardTaxonomiesDict
         return True # no standard taxonomies to test
 
-    def hrefValidForDisclosureSystem(self, href):
+    def hrefValidForDisclosureSystem(self, href) -> bool:
         if self.validTaxonomiesUrl:
             return href in self.validTaxonomiesDict
         elif self.standardTaxonomiesUrl: # fallback to standard taxonomies dict
